;******************************************************************************
;                                                                             *
; Copyright (C) 2012 Sergio García-Cuevas González.                           *
;                                                                             *
; This program is free software; you can redistribute it and/or modify        *
; it under the terms of the GNU General Public License as published by        *
; the Free Software Foundation; either version 3 of the License, or           *
; (at your option) any later version.                                         *
;                                                                             *
; This program is distributed in the hope that it will be useful,             *
; but WITHOUT ANY WARRANTY; without even the implied warranty of              *
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
; GNU General Public License for more details.                                *
;                                                                             *
; You should have received a copy of the GNU General Public License           *
; along with this program.  If not, see <http://www.gnu.org/licenses/>.       *
;                                                                             *
;******************************************************************************

        list      p=12f629
        #include <p12f629.inc>
        #include "asm/config.inc"

        GLOBAL  main_loop
        EXTERN  apply_command,duty,get_command,new_cmd,old_cmd,sGPIO,stable_p,state

MAIN_LOOP       CODE
; Main loop.
main_loop
        CLRWDT                  ; Keep the watchdog on a tight rope.
        BTFSC   state,seen      ; If we have processed the current reading
        GOTO    duty_loop       ; then go to the duty cycle loop
        BSF     state,seen      ; else mark the reading as seen and process it.
        CALL    get_command     ; Read the new command (return is in W).
        MOVWF   new_cmd         ; Store the new command.
        CALL    stable_p        ; Check for a stable reading.
        BTFSS   state,got_cmd   ; If we don't have a stable reading
        GOTO    unstable        ; then update some variables and try again
        BTFSS   state,applied   ; else if we haven't applied the command yet
        CALL    apply_command   ; then we apply it.
        GOTO    main_loop       ; Keep looping.
unstable
        BCF     state,got_cmd   ; Mark the command reading as unstable.
        BCF     state,applied   ; Mark the command as not applied.
        MOVF    new_cmd,W       ; Copy the value of the new command
        MOVWF   old_cmd         ; into the old command.
        GOTO    main_loop       ; Keep looping.
duty_loop
        ; Throttle-down the motors with a simple duty-cycle loop.
        BTFSS   state,motors_on ; If motors are off
        GOTO    main_loop       ; then go back to the main loop
        GOTO    $+1             ; else wait
        GOTO    $+1             ; for 16 microseconds
        GOTO    $+1             ; just to make
        GOTO    $+1             ; the motor switching
        GOTO    $+1             ; frequency a little
        GOTO    $+1             ; longer to make up for
        GOTO    $+1             ; the motor inertia
        GOTO    $+1             ; and reduce the buzzing sound.
        DECFSZ  duty,F          ; Decrease the duty-cycle counter.
        GOTO    main_loop       ; Go back to the main loop if counter > 0.
        BTFSS   state,duty_high ; If we entered the high part of the cycle
        GOTO    high_part       ; then act in consequence
        MOVLW   duty_off        ; else reset the counter
        MOVWF   duty            ; for the low part of the cycle,
        BCF     state,duty_high ; flag as we are on the low part,
        BCF     sGPIO,motors_pin ; turn off the motors
        MOVF    sGPIO,W
        MOVWF   GPIO
        GOTO    main_loop       ; and go back to the main loop.
high_part
        MOVLW   duty_on         ; We entered the high part of the cycle,
        MOVWF   duty            ; so reset the counter,
        BSF     state,duty_high ; flag as we are on the high part,
        BSF     sGPIO,motors_pin ; turn on the motors
        MOVF    sGPIO,W
        MOVWF   GPIO
        GOTO    main_loop       ; and go back to the main loop.

        END
